
# q-expansion of newform 1875.2.a.j, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1875
weight = 2
poly_data = [-1, 3, 29, -1, -11, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[-1, 0, 5, -6, -1, 1], 4], [[-1, 24, 5, -10, -1, 1], 4], [[-13, 24, 9, -10, -1, 1], 4], [[3, 34, -7, -12, 1, 1], 2]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0, 0, 0, 0], [-1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0], [-1, 1, 0, 0, 0, 0], [0, 1, 0, 0, 0, -1], [0, 1, 0, 1, 0, 1], [-3, 0, 1, -1, -1, -1], [3, 0, 1, 2, -1, 0], [-2, -1, 0, -1, -1, 1], [0, 1, 1, 0, 0, 1], [-1, -2, -2, 1, 1, 1], [-4, 1, -1, 0, 0, -1], [-1, -1, -3, -1, -1, -1], [-1, 1, -2, -1, 1, 0], [-4, -2, 0, -3, 1, 0], [-2, 0, 2, 4, 1, 0], [4, -1, 1, 3, 0, 2], [3, -1, -3, 1, -1, 1], [-1, 1, 2, -1, 1, 2], [-3, -1, 1, -3, -2, -1], [-2, 0, -1, 2, 2, -1], [-1, 2, 1, 1, -2, -2], [2, 0, 1, -1, 2, -1], [-5, -1, 3, -8, 2, 0], [0, 3, 2, 0, 1, -2], [-1, -2, -2, 4, 1, 0], [1, -2, -2, -3, 1, 1], [7, 3, 4, 5, -1, 0], [0, 1, 3, 1, -1, 0], [-5, 1, -2, -5, 2, -1], [-4, -4, 0, 5, 1, 0], [-10, 1, 1, 0, 1, 0], [-9, -4, -3, -1, 1, 0], [0, 2, 0, -8, 0, 0], [-10, 3, 1, 1, 1, 0], [9, -1, 3, 4, -1, 3], [-9, 1, 4, 2, 1, -3], [-3, -3, -6, 2, 0, 0], [-7, 3, -3, -5, 2, -3], [0, 1, 4, -1, -4, -2], [-6, -5, -3, 1, 4, 0], [-3, -3, -1, -6, 0, 2], [-3, -4, 0, -1, 1, 1], [0, -1, -2, 1, -4, 1], [-6, 2, 0, 5, -2, -2], [6, -3, 1, 1, 0, 0], [4, -5, -5, 3, 2, -4], [1, -2, 0, -3, 3, 3], [8, -2, -2, 9, -3, -2], [5, -4, -1, 1, 3, 2], [-5, 0, 0, -3, -2, 3], [4, 0, 1, 4, -3, 1], [-4, 0, 5, 0, -2, 2], [2, -4, -1, -2, 1, -1], [7, 2, -2, 4, 1, 5], [-1, -1, 1, -2, -5, 3], [-3, 9, -2, -4, 1, -2], [5, 4, 7, -1, 0, 0], [-5, -3, 0, -3, -2, 3], [-5, 0, 3, -11, 1, -2], [-1, 4, 2, 3, -5, -3], [-4, 3, 3, 3, 3, 3], [-3, 6, 3, -3, -2, -4], [3, 3, 2, 3, 1, 4], [-7, 3, 5, 0, -3, -7], [-4, -4, -2, -5, 3, 6], [1, -4, -6, -4, -4, -1], [5, -2, -2, 6, 2, -3], [2, 4, 1, 12, -1, 1], [-2, 1, -1, 8, -4, 1], [5, -2, 1, 9, 2, 2], [-3, 6, -3, -8, 3, 0], [10, -7, -3, 7, 4, 0], [-13, -8, -4, -6, 0, -1], [-9, 4, -1, 1, -4, -2], [7, 3, 0, 3, -7, 0], [5, -3, -4, 1, 6, -2], [-7, -8, 1, -2, 1, -2], [-1, 6, 0, 4, -1, -5], [-12, -1, -1, -12, 2, -1], [-13, -1, 9, -5, 6, -1], [-14, 10, 3, -4, -2, 2], [-15, 2, -2, 1, -3, -5], [-7, -4, -5, -14, 4, -4], [21, 1, -4, 3, -1, 2], [-17, 3, -1, -18, -1, -1], [-17, -2, 2, -4, -1, 2], [-1, -6, -2, 3, 1, 9], [-5, -2, -8, -5, 4, 4], [10, -4, 1, -7, 6, 3], [5, 1, 6, 0, 0, -2], [-17, -1, 5, 4, 1, -1], [8, 5, 7, 5, 2, 6], [-7, -2, 4, -8, 2, 1], [10, 6, 2, 12, 2, 0], [5, 5, -4, -7, -1, 2], [-16, 0, 0, -4, -3, 3], [1, 0, 4, 23, 0, 1], [5, 0, 2, -10, -4, 1], [3, -4, -8, 11, 2, 6], [17, -7, 1, 4, 1, -1], [-21, -3, -3, -4, -2, -6], [-11, 9, 0, -3, -1, -6], [-3, -2, 1, 6, -4, 3], [-1, -6, 4, -15, 7, 1], [-9, 4, -1, -18, 3, 2], [12, 4, -1, 17, 2, -1], [-10, 0, 7, 5, -7, 0], [1, 1, -10, 0, 2, 0], [-6, 6, -2, -15, -2, -2], [16, -6, -7, 3, 0, -5], [0, -12, -2, 4, -3, 5], [-3, 6, -3, -14, -2, -6], [-1, 1, -7, 10, -5, -5], [-2, 9, 4, 6, 2, 5], [-6, 8, 6, 3, -1, 0], [-2, -2, 1, -12, 5, -3], [-15, 8, 4, -7, 1, -5], [-3, 11, -2, 6, -3, -5], [1, 8, -3, 6, 1, -2], [14, -4, 6, -10, 4, 2], [7, -2, -1, 21, 1, 1], [-21, 3, -4, -4, -4, -2], [10, 1, -3, -12, 3, -4], [7, 2, 0, -14, -2, 3], [3, -1, 2, 6, -1, 6], [4, 0, 7, 8, -6, 1], [-2, 0, -13, -5, -4, -3], [18, 6, 3, 12, -7, -5], [-23, 0, 5, 1, -6, 0], [-1, -5, -1, -3, 4, -9], [-9, 7, 0, -7, -3, 2], [-5, 6, 6, -6, 4, 3], [6, -14, 6, 5, 0, 2], [21, 3, 3, 22, -6, -7], [14, -5, -5, 14, 7, 2], [0, 1, 2, -2, 1, 7], [-9, 1, 0, -7, 1, -6], [-2, 3, -12, 8, 5, 1], [2, -3, -5, -7, -1, 9], [-13, -2, -2, 3, 3, 3], [7, -5, 6, 10, -4, -4], [-15, 1, -4, -3, -7, -6], [-4, -3, 1, -24, 1, 4], [2, 5, -8, 7, -2, 3], [-25, 12, 5, -21, 2, 0], [4, 3, -10, 12, -1, 7], [-16, -5, 1, -19, -2, -4], [18, -13, 4, 21, 1, 7], [-17, -6, 3, -7, 2, -4], [-3, 4, 7, 5, 3, 2], [28, -6, -6, 12, 6, 6], [7, -13, -1, 1, 0, 7], [-4, 9, 0, 9, 3, -3], [-17, -4, 1, -4, 5, 8], [-9, 5, 10, 8, -4, 2], [-11, 11, 12, -7, -1, -2], [9, -7, -4, 0, -1, -11], [-3, -3, 10, -4, 3, 3], [6, -3, -5, 16, -2, -11], [7, -8, -9, 4, 1, 2], [10, -13, 1, -3, 1, -3], [-24, -9, -1, -14, 3, -2], [-6, 7, 3, -6, -5, 6], [16, -1, 0, -9, 0, -3], [2, 3, -1, -4, 7, -2], [-3, 1, 10, -13, -3, -8], [-25, 3, -11, -16, 9, -1], [8, 4, -13, -18, 6, 1], [3, -7, 3, 10, 4, 9], [-3, 8, 5, 7, -8, 4], [-14, 6, 7, -4, -6, -3], [9, 6, 3, 29, 6, -4], [18, 13, 2, 18, 3, 2], [-9, 8, -6, -2, 6, -3], [-19, 13, 9, -24, 0, -6], [-20, 1, 3, 4, -5, -2], [8, 19, 7, 5, -3, -1], [23, 6, 6, -6, -2, -3], [33, -7, 0, 7, 8, 5], [29, -3, -7, -1, -4, 3], [11, 1, -10, -8, -6, 2], [6, 4, -9, -5, 3, -2], [16, -15, 1, 15, -5, 5], [28, 8, 6, 8, -6, 0], [-13, -3, -7, -10, 7, -7], [-27, -2, -7, -20, -2, -7], [22, 4, -3, -6, -7, 3], [4, -9, -7, 11, 1, -5], [-14, 10, -7, 11, -2, -3], [-10, -1, -1, 1, 4, -4], [-2, 2, 5, 1, 6, 7], [20, 5, -12, -4, 4, 5], [29, 10, 13, 8, -6, 2], [3, -1, -12, 2, -10, 0], [2, -9, -5, 12, 2, -4], [-18, 6, 3, -19, 6, -1], [-36, -1, -1, 3, 1, 2], [0, -5, -10, -16, 9, -3], [25, -7, -9, 28, 9, 3], [23, 5, 5, 2, 6, -5], [15, -11, -3, 10, -1, 1], [1, 3, 3, -2, 5, 3], [8, 6, 5, 0, -6, -3], [-4, 3, -4, 8, -10, -7], [-18, -4, 9, 2, 0, -6], [-15, 0, 0, -33, 7, 5], [17, 3, 0, -12, 2, 8], [18, 28, 3, 0, -12, 1], [-19, 4, 3, -11, -8, 2], [-14, -6, 3, -21, -7, -1], [18, -16, 0, 10, -2, 8], [28, -9, 6, 22, 0, 5], [4, 4, -1, -12, -1, -1], [-32, -2, 2, -22, -6, -2], [6, -6, -20, 4, 1, -2], [33, 9, 14, 12, -12, 0], [28, -11, -7, 10, 6, 6], [3, -8, 9, 4, -5, 6], [-8, 8, 4, -17, 6, -3], [17, 6, -1, 9, -9, -3], [26, -8, -3, 13, 6, 7], [11, -6, 2, -22, 6, -9], [-8, 11, -8, -20, 2, -7], [3, 3, -5, -19, -6, 1], [24, -10, -3, 6, 2, 6], [15, -10, 2, 6, -13, 1], [-21, -11, 0, -2, 0, -12], [12, -3, 0, 12, -12, 1], [-30, -2, -8, -4, 8, 0], [-5, 11, -1, -12, -10, -2], [-8, 6, 9, -1, 12, -3], [8, -6, -15, 7, 2, -3], [11, -5, -11, -8, 0, -2], [-1, -7, 1, -16, -3, -7], [32, -4, 1, 16, -3, 3], [6, 9, 16, 13, -2, 6], [-25, -13, -4, 1, 2, 6], [27, 17, 4, 31, 1, -2], [-10, -8, -8, 21, 3, -2], [4, 0, 6, 13, -1, -2], [13, 8, 13, 6, 1, 2], [26, 8, -6, 27, -1, 6], [-13, -1, 9, -21, 1, 1], [24, 4, -2, 21, 2, 4], [-8, -10, 9, 8, 3, 9], [20, 10, 4, 19, 5, 0], [13, 1, -9, 2, -9, -7], [-13, -2, 3, -10, 5, -4], [11, -6, -4, 17, -9, 5], [7, 3, 8, 7, 8, 5], [-2, -6, -10, -30, -5, 7], [34, 6, 10, 10, -2, 6], [8, 12, 1, 7, 3, -3], [-24, 1, -5, -13, -6, -4], [-28, -6, -11, -13, 12, 7], [-11, -5, 0, 4, 11, -3], [15, 4, -2, -4, -8, -1], [-20, -16, -3, -6, 4, 8], [25, 10, 0, -12, -1, -6], [11, -2, -6, -19, 13, -9], [25, 4, -4, -13, -1, 3], [-6, -2, 4, -6, -11, -2], [-13, -3, -3, -8, 8, -7], [25, 21, 2, 1, -4, -3], [35, -5, 9, -3, 0, 7], [-13, 7, 6, 4, -15, -2], [-1, -2, -4, 17, 4, 2], [34, -2, 15, 9, 0, 5], [-21, -2, -5, 10, 6, -10], [-1, -2, 3, 8, 8, -4], [26, 6, -4, -8, 4, 6], [34, 10, 16, 20, 5, 6], [5, -9, -9, -5, -2, -13], [-10, -1, -18, -5, 10, -2], [20, 12, -6, 3, 7, 4], [35, -12, 5, 14, 9, 8], [24, -4, -1, 31, 3, 8], [-32, -8, -4, -28, -3, -3], [-1, -11, -5, 1, -14, 11], [9, 1, -5, 0, -9, 1], [6, -5, -6, 10, -10, 7], [6, -5, -4, 6, 4, 11], [-16, -7, -10, -28, -1, -7], [5, 17, -11, -11, -8, -7], [4, -16, 7, 4, -3, 5], [-18, -3, -5, -19, -3, -10], [7, 3, -5, 12, -4, 14], [19, 5, -5, -8, 1, -11], [15, -11, 3, 10, 2, 9], [3, 9, -6, 8, -5, -13], [-1, -8, 2, 4, 10, -3], [25, -6, -16, -8, 1, -2], [-4, 11, -8, 14, -10, -7], [-28, 13, 20, -12, -7, -5], [9, 5, 9, -31, -9, 11], [26, -17, 0, -6, -6, -3], [24, 5, 12, 10, -11, 8], [-12, -6, 4, 15, -3, 0], [-6, 9, 2, -4, -10, -13], [22, 7, 2, 12, 1, 9], [2, -3, -1, 6, 12, -3], [-7, 1, -11, -12, -3, -17], [-29, -23, -12, -6, 6, 0], [-33, -1, 8, -17, 11, 6], [36, -1, 3, 8, -10, 0], [-2, -3, 0, -6, 6, -13], [22, 11, 20, 8, -7, 0], [18, -18, -12, 14, 2, 4], [-25, -8, 8, -12, 6, 3], [1, -7, -14, -21, 7, 6], [4, 8, -3, -26, -1, -5], [31, -12, 2, 16, -3, 7], [-19, 10, 6, 9, -1, 9], [29, -2, 7, 36, 7, 4], [19, 7, 2, -13, 0, -11], [-20, -18, -5, -2, 5, 3], [-17, -12, 7, -9, -4, -12], [-38, 11, 17, -26, 2, -5], [10, 13, -9, 4, 6, 14], [-6, -8, -1, 8, 1, -11], [-21, -5, -7, -10, -4, 7], [20, 2, 3, 9, 1, 7], [10, 9, -11, 16, -9, 0], [-1, -5, 1, 19, -7, 4], [34, 16, 10, 23, 8, -5], [-18, -10, 0, -11, -5, 0], [34, -3, -13, -12, -5, 8], [37, 3, 7, 45, -18, -7], [-7, 19, 0, -15, 2, 0], [-28, -18, -1, -20, -2, -9], [-44, 5, -18, -9, 8, -7], [4, -17, 6, -2, 0, 5], [19, -4, 16, 8, 12, 11], [1, 5, 8, 28, 0, 12], [8, -3, -2, 9, -7, -7], [-14, 24, -4, 0, -1, -11], [44, -15, -1, 0, -2, 4], [3, 0, 3, -8, -2, 18], [19, 5, -14, 13, -5, 6], [7, 20, 14, -3, -10, -4], [27, 11, 17, 4, -11, -5], [-15, -1, 4, -3, 2, 4], [-16, 28, -2, -8, 2, -14], [-11, -5, -6, 5, 6, 5], [-15, 11, -4, 16, 0, -6], [2, 11, 3, 22, 2, -1], [36, -3, 14, 20, 6, 13], [-36, 1, 6, -21, 1, -7], [-5, 9, 13, -22, -10, 11], [-22, -12, 1, -21, 0, 1], [-18, -14, 4, -20, 9, 6], [21, 13, -12, 5, 0, 9], [1, 11, -2, 20, -8, -2], [-27, 3, 1, -2, -13, -15], [-10, -8, 7, 33, -5, 7], [13, 6, 9, 16, -13, 2], [-8, 18, 3, -26, 3, -9], [-2, 7, 18, 15, -8, 14], [4, -6, 15, 17, 0, 1], [-11, 1, 13, -2, -14, -12], [-32, 6, 0, -20, -13, -5], [-17, 4, 0, -53, 3, -1], [17, 0, -6, 23, -5, 3], [7, -5, 6, -13, -1, 8], [12, -1, 15, 32, -6, 11], [19, -20, -16, 15, 6, 11], [-36, 6, 18, -7, 11, 2], [24, -9, 2, 31, 2, 0], [-41, -11, 3, -5, 6, -5], [-54, 11, 7, -56, 1, -2], [0, -18, -5, -6, 19, -5], [-8, -14, -17, -30, 5, -13], [15, -11, -15, -5, 8, 21], [-39, -3, -16, -18, 6, -6], [-14, 16, 14, 17, 7, 2], [4, 2, 9, -13, -12, -5], [13, -6, 16, -3, -6, 4], [8, 10, 24, -2, -11, -5], [-26, 10, -5, -38, -2, 0], [-23, 25, -2, -33, 3, -6], [-13, -12, -21, 18, 14, -7], [-64, 10, 8, -8, 0, 4], [-14, -13, -14, -9, 6, 9], [8, 15, -2, -18, -8, -1], [-13, 5, -2, 23, -3, 4], [2, -10, 7, 14, -17, -9], [-20, 3, 5, 10, 0, 3], [-7, 22, 23, -21, -6, -16], [24, 9, -1, -4, -7, -14], [13, -9, 3, -12, 12, -1], [-11, 11, 15, 21, 6, -11], [-12, -2, -1, 2, 13, -9], [-12, -10, -10, 21, 11, -16], [-25, -8, -24, 2, 14, -1], [0, -22, -11, 20, 14, 1], [14, 8, 3, -19, -2, 13], [-6, -4, 2, 11, 6, -9], [-14, 27, -3, -1, 10, 0], [35, 10, 19, 23, 2, 6], [-8, 9, 12, -4, -11, 0], [-5, 10, 0, 14, 13, -1], [5, 5, 2, -12, 12, -12], [10, 22, 18, 6, -3, 10], [13, 20, 7, 21, 12, 2], [13, -15, -15, 16, 11, -11], [-14, -1, 5, 11, -17, -2], [-16, 2, -8, 8, -7, -10], [31, 27, 12, 2, -6, -4], [28, -8, -12, 30, -12, 4], [3, -19, 0, 17, -5, 6], [-25, -2, -4, 19, -1, 7], [-3, 6, -24, -15, 11, -3], [5, 2, -7, 4, 17, -6], [3, -3, -12, -29, -6, 6], [38, -21, -8, 4, -2, 3], [31, -16, -7, 44, 9, 4], [-21, -10, 0, -43, 11, 1], [1, -7, -1, -16, 7, 9], [12, 15, -10, 3, 13, 9], [15, 19, -1, 35, 11, 1], [29, -18, -5, 0, 0, 7], [-37, -10, -17, 6, 3, 0], [-4, -16, -2, -20, 0, 4], [-27, 6, -15, 2, -2, -13], [15, -12, -2, 22, 6, -7], [0, -19, 15, 32, -1, 2], [-10, 7, 13, -19, 11, -4], [12, -13, -21, 27, 0, 16], [-5, 19, 3, 8, 1, 11]]
